"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const async_1 = require("async");
const debug_1 = __importDefault(require("debug"));
const ramda_1 = require("ramda");
const uuid_1 = require("uuid");
const debug = debug_1.default("rascal:ProxyPublication");
class PublicationProxy {
    constructor(name, broker) {
        this.messageMap = new Map();
        this.INTERVAL_PERIOD_SECOND = 10;
        this.name = name;
        this.broker = broker;
        this.failedQuene = this.createFailedMessageQueue();
        this.pauseQueue();
    }
    get defaultPushOptions() {
        return {
            options: {
                messageId: uuid_1.v4()
            }
        };
    }
    publish(message, overrides) {
        return __awaiter(this, arguments, void 0, function* () {
            const config = ramda_1.mergeDeepRight(this.defaultPushOptions, overrides || {});
            this.addMessage(config.options.messageId, arguments);
            const emiter = yield this.broker.publish(this.name, message, config);
            this.attachPublishHandlers(emiter);
            return emiter;
        });
    }
    pauseTimer() {
        if (this.interval) {
            clearInterval(this.interval);
            this.interval = undefined;
        }
    }
    runTimer() {
        if (!this.interval) {
            this.interval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
                const queueHead = this.failedQuene._tasks.head;
                if (queueHead) {
                    yield this.retryByMessageId(queueHead.data);
                }
            }), this.INTERVAL_PERIOD_SECOND * 1000);
        }
    }
    pauseQueue() {
        if (!this.failedQuene.paused) {
            this.failedQuene.pause();
            this.runTimer();
        }
    }
    resumeQueue() {
        if (this.failedQuene.paused) {
            this.pauseTimer();
            this.failedQuene.resume();
        }
    }
    attachPublishHandlers(publihser) {
        publihser.on("success", this.onConfirmSuccess.bind(this));
        publihser.on("return", this.onConfirmReturn.bind(this));
        publihser.on("error", this.onConfirmError.bind(this));
    }
    createFailedMessageQueue() {
        return async_1.queue((messageId, callback) => __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.retryByMessageId(messageId);
                callback();
            }
            catch (error) {
                callback(error);
            }
        }), 1);
    }
    retryByMessageId(messageId) {
        return __awaiter(this, void 0, void 0, function* () {
            const args = this.messageMap.get(messageId);
            if (!args) {
                return;
            }
            yield this.publish.apply(this, args);
        });
    }
    queueContains(messageId) {
        const tasks = this.failedQuene._tasks.toArray();
        return tasks.includes(messageId);
    }
    pushQueue(messageId) {
        if (this.queueContains(messageId)) {
            return;
        }
        this.failedQuene.push(messageId, (err) => {
            if (err) {
                debug("Message retry Error: %s for msgId %s", err.message, messageId);
            }
        });
    }
    onConfirmError(error, messageId) {
        return __awaiter(this, void 0, void 0, function* () {
            debug("Message confirm Error: %s for msgId %s", error.message, messageId);
            this.pauseQueue();
            this.pushQueue(messageId);
        });
    }
    onConfirmSuccess(messageId) {
        return __awaiter(this, void 0, void 0, function* () {
            debug("Message confirm success: %s", messageId);
            this.delteMessage(messageId);
            this.resumeQueue();
        });
    }
    onConfirmReturn(message) {
        return __awaiter(this, void 0, void 0, function* () {
            debug("Message confirm return: %s", message.properties.messageId);
            this.delteMessage(message.properties.messageId);
        });
    }
    addMessage(messageId, message) {
        this.messageMap.set(messageId, message);
    }
    delteMessage(messageId) {
        this.messageMap.delete(messageId);
    }
}
exports.default = PublicationProxy;
